// Address.hxx
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(ADDRESS_HXX__INCLUDED) // {
#define ADDRESS_HXX__INCLUDED

#include "basic.hxx"

namespace Rescue
{
	/////////////////////////////////////////////////////////////////////////
	// Address

	/**
	* IP Opposite of address and port number.
	*/
	class Address
	{
	private:

	protected:
		/** IP address of network byte order */
		u_long m_address;
		/** Port number of network byte order */
		u_short m_port;
		/** Validity? */
		u_short m_isValid;
		/** Validity? */
		bool isValid() const;

	public:
		bool operator== (const Address& rhs) const;
		bool operator< (const Address& rhs) const;

		virtual ~Address();
		/** Invalid host name */
		class IllegalHost : public std::runtime_error {
		public:	IllegalHost(const std::string& what_arg) : runtime_error(what_arg) { }
		};
		/** It initializes as invalid Address */
		Address();
		/**
		* It initializes as effective Address
		* @param	address
		*				IP address of byte order of host side
		* @param	port
		*				Port number of byte order of host side
		* @param	dummy
		*				Always true. When Address (0, foo) is, you use for the case where the compiler is confused.
		*/
		Address(u_long address, int port, bool dummy = true);
		/**
		* It initializes as effective Address
		* @param	name
		*				Name of node. " Localhost" And " 127.0.0.1" And so on.
		* @param	port
		*				Port number of byte order of host side
		* @throws	IllegalHost
		*				Host could not be converted to IP address.
		*/
		Address(const char* name, int port); //throw(IllegalHost)

		/** IP address of byte order of host side */
		u_long address() const;
		/** IP address of network byte order */
		u_long addressAsNetworkOrder() const;
		/** Port number of byte order of host side */
		u_short port() const;
		/** Port number of network byte order */
		u_short portAsNetworkOrder() const;
		
	private:
		/*Address(const Address& source);
		Address& operator= (const Address& rhs);
		bool operator== (const Address& rhs) const;*/
	};
	
	/////////////////////////////////////////////////////////////////////////
} // namespace Rescue

#ifdef NDEBUG
#  include "Address.inl"
#endif

#endif // } !defined(ADDRESS_HXX__INCLUDED)

