// AppTechTrainning.cxx
//
/////////////////////////////////////////////////////////////////////////////

#include <iostream.h>

#include "common.hxx"
#include "AppTech.hxx"
#include "AppTechObjectPool.hxx"
#include "AppTechFlammableBuilding.hxx"


namespace Rescue
{
	/*******************************************************************************
	!
	! Calculation of fire fighting power effect (p (t) part)
	!
	!
	!*******************************************************************************/
	// ! Fire fighting power effective calculation function (t_gfr: Fire weight)
	static double syobo_p(const  AppTechFlammableBuilding & fb)
	{
		double pr_syoka  = 60.0; // ! Pr_syoka: Extinguishing quantity per water 1t (kg/t)
		//!now().water : Per  t drainage quantity
		return (pr_syoka * fb.now.water) / fb.fire.gfr_rst;
	}

        /*******************************************************************************
	!
	!
	! Fire fighting power decision function (calculation of fire fighting power repletion rate)
	!
	!
	!*******************************************************************************/
	//! Fire fighting power decision function (the building 1f area and the number of throwing 
       //nozzle heads are designated as argument and fire fighting power repletion rate is returned)
	static double s_hantei(const AppTechFlammableBuilding & fb)
	{
		//! Argument
		//real      :: tmen1f      ! Building 1f area
		//type(tmp) :: now(kensu)  ! Main several drainage quantities of nozzle head
		//! Local variable
		//real    :: syutyo      ! Circumference of building
		//real    :: tutu		 ! Number of the necessary nozzle heads
			
		double syutyo= 4.5 * sqrt(fb.areaGround());	//! Calculating the circumference of the building
		double tutu  = syutyo / 10.0;				//! Number of the necessary calculating nozzle heads
		double res  = double(fb.now.tutu) / tutu;		// ! Fire fighting power repletion rate it returns at ratio and makes value


		return (res >= 1)? 1: res;
	}

	/*******************************************************************************
	!
	!
	! Calculation of fire fighting power effect (propagation factor part)
	!
	!
	!*******************************************************************************/
	//! Fire fighting power effective calculation function
	static double syobo_t(const AppTechFlammableBuilding & fb)
	{
		double res = 1.0 - s_hantei(fb); //! Repletion rate decision
		return (res >= 1)? 1: res;
	}

	/*******************************************************************************
	!
	!
	! 16 directions -> radian conversion
	!
	!
	!*******************************************************************************/
	static double rad(const int i) //! Unit conversion
	{
		double res = M_PI * (1.5 - double(i) / 8.0);       //! Value of wind direction (kisyo.ndw) from angle (rad) it calculates
		while (res > 2 * M_PI) {
			res -= 2 * M_PI;
		}
		while (res < 0) {
			res += 2 * M_PI;
		}
		return res;
	}
	/*******************************************************************************
	!
	!
	! Radian -> 16 direction conversions
	!
	!
	!*******************************************************************************/
	static int m16(const double x) //! Unit conversion
	{
		int k = int(x/(2 * M_PI / 16)); 
		int res = 12 - k;
		while (res <= 0) {
			res += 16;
		}
		while (res > 16) {
			res -= 16;
		}
		return res;
	}

	void AppTechObjectPool::dataread()  
	{
		Objects::const_iterator it_objects = objects().begin();
		for (; it_objects != objects().end(); it_objects++) {
			AppTechFlammableBuilding* fb = dynamic_cast<AppTechFlammableBuilding*> (*it_objects);
			if (fb != 0) {
				fb_v.push_back(fb);
			}
		}
		std::vector<AppTechFlammableBuilding*>::const_iterator it_tikudat =  fb_v.begin();
		for (int i = 0; it_tikudat != fb_v.end(); it_tikudat++,i++){
			(*it_tikudat)->tikudat_index = i;
		}

		int s = tikudat.size();
		hensu_MP_nkk = s;
		for (int i = 0; i < s; i++){
			double wkf;
			switch (tikudat[i].m_buildingCode_mkdat){
			case 11:
				tikudat[i].k2  = 1;
				wkf = 120.0;
				break;
			case 21:
				tikudat[i].k2  = 2;
				wkf = 120.0;
				break;
			default:
				tikudat[i].k2  = 3;
				wkf = 30.0;
			}
			fire[i].gfr_ttl  = tikudat[i].m_dblBuildingAreaTotal  * wkf ;
			fire[i].p  = 1.0/(1.0 + exp(-1.0*para.pra *para.prc ));
			fire[i].gfr_rst  = fire[i].gfr_ttl  - fire[i].p *fire[i].gfr_ttl ;
		}
	}

	void AppTechObjectPool::calc(const S32 time)
	{
		icall++;
		tnow = time * 60;

		int  Tmp1;
		int  Tmp2;
		//double  Tmp3;
		int  Tmp4;
		//int  Tmp5;
		auto double  ctt3;
		auto double  cw;
		auto double  rft[(tikudat.size() - 1) ];
		auto double  rxt[(tikudat.size() - 1) ];
		 
		auto int  i;
		auto int  ichk = 1;
		auto int  iend_flag;
		register int  j;
		auto int  nnn;
		register int  isimyu;
		register int  ke;
		auto int  kz;
		auto int  t;
		auto int  kd;
		auto int  ipara;
		register int  k;
		register int  l;
		auto int  ifg;
		auto double  x;
		auto double  y;
		auto double  tsita;
		auto double  water;
		auto double  ang;
		auto double  pv;
		auto double  wkf;
		auto double  gfr;
		auto double  syobo_hi;
		auto double  rest;
		auto double  rsss;
		auto double  ss;
		auto double  p1;
		auto double  p;
		auto double  hhk;
		auto double  fpw;
		auto double  ffpw;
		auto double  sita;
		auto double  psita;
		auto double  dd;
		auto double  hh;
		auto double  hdd;
		auto double  ann;
		auto double  s;
		auto double  bb;
		auto double  hoko;
		auto double  tran;
		auto double  ag;
		auto double  hen;
		auto double  ff;
		auto double  ct1;
		auto double  ct2;
		auto double  ct3;
		auto double  ctt1;
		auto double  ctt2;

		{ cout << " " <<  "check write Start"  ;} ;
		{ cout << " " <<  icall   ;} ;
		{ cout << " " <<  " Time eye"  ;} ;
		{ cout << endl; cout.flush();} ;
		if (icall == 1) {

			{ cout << " " <<  "//////////////////////////////////////////////////"  ;} ;
			{ cout << endl; cout.flush();} ;
			if (ipara == 1) {

				{ cout << " " <<  " Note note note **** ignition decision value was read from the parameter file! **** note note note"  ;} ;
				{ cout << endl; cout.flush();} ;
			}

			{ cout << " " <<  " As for building number of cases"  ;} ;
			{ cout << " " <<  hensu_MP_nkk  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "//////////////////////////////////////////////////"  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  " Spread of fire parameter"  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_tsl(1)"  ;} ;
			{ cout << " " <<  para .tsl [1 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_tsl(2)"  ;} ;
			{ cout << " " <<  para .tsl [2 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_tsl(3)"  ;} ;
			{ cout << " " <<  para .tsl [3 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_ts_1(1)"  ;} ;
			{ cout << " " <<  para .ts1 [1 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_ts_1(2)"  ;} ;
			{ cout << " " <<  para .ts1 [2 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_ts_1(3)"  ;} ;
			{ cout << " " <<  para .ts1 [3 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_cww(1)"  ;} ;
			{ cout << " " <<  para .cww [1 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_cww(2)"  ;} ;
			{ cout << " " <<  para .cww [2 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_cww(3)"  ;} ;
			{ cout << " " <<  para .cww [3 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;
			 
			 
			 

			{ cout << " " <<  "calc_arg_para%at_fprmt_ctw(1)"  ;} ;
			{ cout << " " <<  para .ctw [1 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_ctw(2)"  ;} ;
			{ cout << " " <<  para .ctw [2 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_ctw(3)"  ;} ;
			{ cout << " " <<  para .ctw [3 + -1]  ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_prc"  ;} ;
			{ cout << " " <<  para .prc   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_pra"  ;} ;
			{ cout << " " <<  para .pra   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_prv1"  ;} ;
			{ cout << " " <<  para .prv1   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_sta_gnd"  ;} ;
			{ cout << " " <<  para .sta1   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_sta_all"  ;} ;
			{ cout << " " <<  para .stal   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_stf1"  ;} ;
			{ cout << " " <<  para .stf1   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_stqw"  ;} ;
			{ cout << " " <<  para .stqw   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_sts"  ;} ;
			{ cout << " " <<  para .sts   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_std"  ;} ;
			{ cout << " " <<  para .std   ;} ;
			{ cout << endl; cout.flush();} ;

			{ cout << " " <<  "calc_arg_para%at_fprmt_dt"  ;} ;
			{ cout << " " <<  para .dt   ;} ;
			{ cout << endl; cout.flush();} ;
			for(i = 1;i <= (tikudat.size() - 1) ;i++) {
				enslog [i + -1].start  = -1;
				enslog [i + -1].time1  = -1;
				enslog [i + -1].time2  = -1;
				enslog [i + -1].aend  = -1;
			}
		}

		{ cout << " " <<  " Current time"  ;} ;
		{ cout << " " <<  tnow   ;} ;
		{ cout << " " <<  "Second"  ;} ;
		{ cout << endl; cout.flush();} ;
		if (ichk == 1) {

			{ cout << " " <<  "0 variables are initialized"  ;} ;
			{ cout << endl; cout.flush();} ;
		}
		iend_flag = 0;
		for(i = 1;i <= (tikudat.size() - 1) ;i++) {
			rft[i + -1] = 0.f;
			rxt[i + -1] = 0.f;
			now[i + -1].tutu  = 0;
			now[i + -1].water  = 0.f;
			 
			fire [i + -1].ikiken  = 0;
		}
		if (ichk == 1) {

			{ cout << " " <<  "It rearranges number of 0 nozzle heads"  ;} ;
			{ cout << endl; cout.flush();} ;
		}
		 
		Tmp1 = in_tutu.size();
		for(i = 1;i <= Tmp1;i++) {
			if (ichk == 1) {

				{ cout << " " <<  "1 nozzle head id is searched"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			Tmp2 = hensu_MP_nkk;
			for(j = 1;j <= Tmp2;j++) {
				if (in_tutu [i + -1].building->id() == tikudat [j + -1].id() ) {
					nnn = j;
					goto CONSTRUCT_1_EXIT;
				}
			}
		CONSTRUCT_1_EXIT:;
			if (ichk == 1) {

				{ cout << " " <<  "2 Throwing direction of  nozzle heads is calculated"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			x = (in_tutu [i + -1].xy [1 + -1] - tikudat [nnn + -1].ajushin .x() )* 1.0 / 1000;
			y = (in_tutu [i + -1].xy [2 + -1] - tikudat [nnn + -1].ajushin .y() )* 1.0 / 1000;
			if (ichk == 1) {

				{ cout << " " <<  "3 throwing directions seek some quadrant"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			if ((x>=0) && (y>=0)) {
				tsita = (double )(atan(y/x));
			} else if ((x<0) && (y>=0)) {
				tsita = (double )(atan(y/x)) + M_PI ;
			} else if ((x<0) && (y<0)) {
				tsita = (double )(atan(y/x)) + M_PI ;
			} else if ((x>=0) && (y<0)) {
				tsita = (double )(atan(y/x));
			}
			if (ichk == 1) {

				{ cout << " " <<  "4 Correcting in 16 directions, it substitutes to the structure"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			in_tutu [i + -1].ewsn  = m16(*( &tsita )) ;
			if ((in_tutu [i + -1].ewsn <1) || (in_tutu [i + -1].ewsn >16)) {

				{ cout << " " <<  "error  Direction of the nozzle head is illegitimate"  ;} ;
				{ cout << " " <<  i  ;} ;
				{ cout << " " <<  " As for flat knot direction"  ;} ;
				{ cout << " " <<  in_tutu [i + -1].ewsn   ;} ;
				{ cout << endl; cout.flush();} ;

				{ cout << " " <<  " As for value"  ;} ;
				{ cout << " " <<  in_tutu [i + -1].ewsn   ;} ;
				{ cout << endl; cout.flush();} ;
				return ;
			}
			if (ichk == 1) {

				{ cout << " " <<  "5 Drainage quantitative setting per unit time"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			water = in_tutu [i + -1].quantity / 60 * para .dt ;
			if (ichk == 1) {

				{ cout << " " <<  "6 Number of drainage quantitative nozzle heads classified by building is totaled"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			now[nnn + -1].tutu  = now[nnn + -1].tutu  + 1;
			now[nnn + -1].water  = now[nnn + -1].water  + water;
		}

		{ cout << " " <<  " Building number, number of nozzle heads, drainage quantity (t)"  ;} ;
		{ cout << endl; cout.flush();} ;
		Tmp1 = hensu_MP_nkk;
		for(i = 1;i <= Tmp1;i++) {
			if (now[i + -1].tutu !=0) {

				{ cout << " " <<  i  ;} ;
				{ cout << " " <<  now[i + -1].tutu   ;} ;
				{ cout << " " <<  now[i + -1].water   ;} ;
				{ cout << endl; cout.flush();} ;
			}
		}
		if (ichk == 1) {

			{ cout << " " <<  "7 The meteorological data are set"  ;} ;
			{ cout << endl; cout.flush();} ;
		}
		if ((kisyo .at_ks_ndw<0) || (kisyo .at_ks_ndw>1295999)) {

			{ cout << " " <<  "    error As for wind direction please appoint 0 - 1295999 seconds! ! ! As for present wind direction"  ;} ;
			{ cout << " " <<  kisyo .at_ks_ndw  ;} ;
			{ cout << endl; cout.flush();} ;
			return ;
		}
		if (kisyo .at_ks_wvv<=1.0) {
			kisyo .at_ks_wvv = 1.0;
		}
		ang = kisyo .at_ks_ndw + (90.0 * 3600.0);
		if (ang >= (360.0 * 3600.0)) {
			ang = ang - (360.0 * 3600.0);
		}
		ang = (360.0 * 3600.0) - ang;
		ang = ang * M_PI / (180.0 * 3600.0);
		if (ichk == 1) {
			{ cout << " " <<  "9 It revises wind velocity"  ;} ;
			{ cout << endl; cout.flush();} ;
		}
		pv = pow( (kisyo .at_ks_wvv / 1.0) ,  para .prv1  ) ;
		if (ichk == 1) {

			{ cout << " " <<  "10 Combustion calculation of the building is started"  ;} ;
			{ cout << endl; cout.flush();} ;
		}
		for(isimyu = 1;isimyu <= 1;isimyu++) {
			Tmp1 = hensu_MP_nkk;
			for(ke = 1;ke <= Tmp1;ke++) {
				rft[ke + -1] = 0;
				rxt[ke + -1] = 0;
			}
			if (isimyu!=1) {
				tnow  = tnow  + 30;
			}
			Tmp1 = hensu_MP_nkk;
			for(i = 1;i <= Tmp1;i++) {
				enslog [i + -1].structid  = tikudat [i + -1].id() ;
				if ((fire [i + -1].rank_f == 0) || (fire [i + -1].rank_f == 4)) {
					goto LOOP1_CYCLE;
				}
				switch (tikudat [i + -1].k2 ) {
				case 1:
					wkf = 120;
					kz = 1;
					break;
				case 2:
					wkf = 120;
					kz = 2;
					break;
				case 3:
					wkf = 30;
					kz = 3;
					break;
				default:

					{ cout << " " <<  "Building structural calc_arg_tikudat (I) %at_bld_dat_buildingCode_k2 is illegitimate"  ;} ;
					{ cout << " " <<  tikudat [i + -1].k2   ;} ;
					{ cout << endl; cout.flush();} ;
				}
				gfr = tikudat [i + -1].m_dblBuildingAreaTotal / para .stal *wkf/ para .stqw ;
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "11 Accumulating the reduction combustion weight ratio with fire fighting powers"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				syobo_hi = (double )(syobo_p((tikudat[( i )-1])) );
				fire [i + -1].syobo_sum  = fire [i + -1].syobo_sum  + syobo_hi;
				t = tnow  - fire [i + -1].on_time ;
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "12 The advance quantity of hour ratios is calculated"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				rest = para.tsl[kz + -1] * 60;
				rsss = rest * pv * (double )(sqrt(tikudat [i + -1].m_dblBuildingAreaTotal / para .stal ));
				if (rsss == 0.f) {
					{ cout << " " <<  "It is thought that rsss calc_arg_para%at_fprmt_sta_all which division is done 0 is with 0,"  ;} ;
					{ cout << " " <<  para .stal   ;} ;
					{ cout << endl; cout.flush();} ;
					return ;
				}
				ss = para .dt /rsss;
				fire [i + -1].tpp  = fire [i + -1].tpp  + ss;
				if ((ichk == 1) && (isimyu == 1)) {
					{ cout << " " <<  "13 combustion ratios p (t) it calculates"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				p1 = -(1.0 * para .pra *(fire [i + -1].tpp  + para .prc ));
				p = (1.0 - fire [i + -1].syobo_sum ) / (1.0 + (double )(exp(p1)));
				if ((ichk == 1) && (isimyu == 1)) {
					{ cout << " " <<  "14 remains weight is calculated"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				fire [i + -1].dp  = p - fire [i + -1].p ;
				fire [i + -1].p  = p;
				fire [i + -1].gfr_rst  = fire [i + -1].gfr_rst  - fire [i + -1].gfr_ttl * fire [i + -1].dp ;
				fire [i + -1].a2_lost  = tikudat [i + -1].m_dblBuildingAreaTotal *((fire [i + -1].gfr_ttl  - fire [i + -1].gfr_rst )/ fire [i + -1].gfr_ttl );
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  " 15 spread of fire powers fh are calculated"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				hhk = tikudat [i + -1].floors() * 3.0 / para .stf1 ;
				if (hhk > 3.0) {
					hhk = 3.0;
				}
				if (kz<=2) {
					hhk = hhk * 0.7;
				}
				hhk = (double )(sqrt(hhk));
				fpw = gfr * fire [i + -1].dp *hhk*(double )(sqrt(tikudat [i + -1].m_dblBuildingAreaGround / para .sta1 ));
				if (kz == 3) {
					fpw = fpw * 0.1;
				}
				ffpw = fpw;
				fire [i + -1].rank_f  = 1;
				if (fire [i + -1].tpp > 0.33) {
					fire [i + -1].rank_f  = 2;
				}
				if (fire [i + -1].tpp > 0.67) {
					fire [i + -1].rank_f  = 3;
				}
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "16 Arrival time of combustion time ratio is input"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				if ((fire [i + -1].tpp > 0.33) && (enslog [i + -1].time1 == -1)) {
					enslog [i + -1].time1  = tnow ;
				}
				if ((fire [i + -1].tpp > 0.67) && (enslog [i + -1].time2 == -1)) {
					enslog [i + -1].time2  = tnow ;
				}
				if ((ichk == 1) && (isimyu == 1)) {
					{ cout << " " <<  "17 Extinguishment decision is done"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				if ((fire [i + -1].tpp > 0.98) || (fire [i + -1].dp <=syobo_hi)) {
					if (isimyu == 1) {

						{ cout << " " <<  tikudat [i + -1].id()   ;} ;
						{ cout << " " <<  " Suppressed"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					fire [i + -1].tpp  = 1.0;
					fire [i + -1].rank_f  = 4;
					if (enslog [i + -1].aend == -1) {
						enslog [i + -1].aend  = tnow ;
					} else {

						{ cout << " " <<  "Compacting decision was done to compacting time having entered"  ;} ;
						{ cout << " " <<  i  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					if (fire [i + -1].gfr_ttl == 0) {
						{ cout << " " <<  "With 0 division calc_arg_fire which is done () %at_ensho_gfr_ttl"  ;} ;
						{ cout << " " <<  i  ;} ;
						{ cout << " " <<  tikudat [i + -1].id()   ;} ;
						{ cout << endl; cout.flush();} ;
					}
					if (1.0 - fire [i + -1].gfr_rst / fire [i + -1].gfr_ttl < 0.2) {
						fire [i + -1].rank_w  = 1;
					}
					if ((1.0 - fire [i + -1].gfr_rst / fire [i + -1].gfr_ttl >= 0.2) && (1.0 - fire [i + -1].gfr_rst / fire [i + -1].gfr_ttl < 0.7)) {
						fire [i + -1].rank_w  = 2;
					}
					if (1.0 - fire [i + -1].gfr_rst / fire [i + -1].gfr_ttl >= 0.7) {
						fire [i + -1].rank_w  = 3;
					}
					enslog [i + -1].fstat  = fire [i + -1].rank_w ;
				}
				if ((ichk == 1) && (isimyu == 1)) {
					{ cout << " " <<  "18 Influence computation to contiguity buildings is done"  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				Tmp2 = tikudat [i + -1].nrinsetu.size() ;
				for(j = 1;j <= Tmp2;j++) {
					if ((ichk == 1) && (isimyu == 1)) {
						{ cout << " " <<  "19 The prospect angular setting which excludes outside conditions"  ;} ;
						cout << " " << j;
						{ cout << endl; cout.flush();} ;
					}
					if (ffpw<=0.f) {
						goto LOOP2_EXIT;
					}
					if (fire [i + -1].dp <=syobo_hi) {
						goto LOOP2_EXIT;
					}
					kd = rindat [i  - 1][j - 1].fb->tikudat_index + 1 ;
					if (fire [kd + -1].rank_f !=0) {
						goto LOOP2_CYCLE;
					}
					sita = rindat [i  - 1][j - 1].end_angle  - rindat [i  - 1][j - 1].start_angle ;
					sita = (double )(fabs(sita));
					if (sita> M_PI ) {
						sita = 2* M_PI  - sita;
					}
					if (sita == 0) {

						{ cout << " " <<  "    error -203 prospect angle is 0! ! !"  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  i  ;} ;
						{ cout << " " <<  " Building of eye"  ;} ;
						{ cout << " " <<  j  ;} ;
						{ cout << " " <<  " Contiguity building of turn eye"  ;} ;
						{ cout << " " <<  " Angle 1"  ;} ;
						{ cout << " " <<  rindat [i  - 1][j - 1].start_angle   ;} ;
						{ cout << " " <<  " Angle 2"  ;} ;
						{ cout << " " <<  rindat [i  - 1][j - 1].end_angle   ;} ;
						{ cout << endl; cout.flush();} ;
						return ;
					}
					if (sita> M_PI ) {

						{ cout << " " <<  "    error -204 Prospect angle exceeds  (3.14159)! ! !"  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  i  ;} ;
						{ cout << " " <<  " Building of eye"  ;} ;
						{ cout << " " <<  j  ;} ;
						{ cout << " " <<  " Contiguity building of turn eye"  ;} ;
						{ cout << " " <<  "Angle 1"  ;} ;
						{ cout << " " <<  rindat [i  - 1][j - 1].start_angle   ;} ;
						{ cout << " " <<  "Angle 2"  ;} ;
						{ cout << " " <<  rindat [i  - 1][j - 1].end_angle   ;} ;
						{ cout << endl; cout.flush();} ;
						return ;
					}
					if ((ichk == 1) && (isimyu == 1)) {

						{ cout << " " <<  "20 Computation of propagation power"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					psita = (sita/ para .sts );
					dd = rindat [i  - 1][j - 1].min_dist ;
					hhk = tikudat [kd + -1].floors() * 3.0;
					if ((fire [i + -1].itoukai == 0) && (fire [kd + -1].itoukai == 0)) {
					} else if ((fire [i + -1].itoukai !=0) && (fire [kd + -1].itoukai == 0)) {
						if (tikudat [i + -1].k2 <= 2) {
							dd = dd - 1.8;
						}
					} else if ((fire [i + -1].itoukai == 0) && (fire [kd + -1].itoukai !=0)) {
						if (tikudat [kd + -1].k2 <= 2) {
							dd = dd - 1.8;
						}
						if (tikudat [kd + -1].k2 <= 2) {
							hhk = hhk * 0.7;
						}
					} else {
						if (tikudat [i + -1].k2 <= 2) {
							dd = dd - 1.8;
						}
						if (tikudat [kd + -1].k2 <= 2) {
							dd = dd - 1.8;
						}
						if (tikudat [kd + -1].k2 <= 2) {
							hhk = hhk * 0.7;
						}
					}
					dd = dd/ para .std ;
					if (dd < 0.7) {
						dd = 0.7;
					}
					if (hhk > 12.0) {
						hhk = 12.0;
					}
					hh = hhk / para .stf1 ;
					hdd = hh / dd;
					if ((ichk == 1) && (isimyu == 1)) {
						{ cout << " " <<  "21 Computation of directivity"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					if (rindat [i  - 1][j - 1].start_angle > rindat [i  - 1][j - 1].end_angle ) {
						rindat [i  - 1][j - 1].start_angle  = rindat [i  - 1][j - 1].start_angle  - 2* M_PI ;
					}
					ann = 5.0 * (rindat [i  - 1][j - 1].start_angle  + rindat [i  - 1][j - 1].end_angle );
					s = (ann - ang);
					bb = 1.0 - pv;
					if (1.0 - bb*(double )(cos(s)) == 0) {
						{ cout << " " <<  "With 0 division 1.0-bb*cos which is done (s) bb="  ;} ;
						{ cout << " " <<  bb  ;} ;
						{ cout << " " <<  "cos(s)="  ;} ;
						{ cout << " " <<  (double )(cos(s))  ;} ;
						{ cout << endl; cout.flush();} ;
						return ;
					}
					hoko = 1.0 / (1.0 - bb*(double )(cos(s)));
					if (kisyo .at_ks_wvv <= 1) {
						hoko = 1.0;
					}
					if ((ichk == 1) && (isimyu == 1)) {
						{ cout << " " <<  "22 Setting of propagation factor"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					tran = psita*hoko*hdd*(double )(syobo_t((tikudat[( i )-1])) );
					if ((ichk == 1) && (isimyu == 1)) {
						{ cout << " " <<  "23 The defense effect due to the throwing direction of fire fighting power"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					if (now[i + -1].tutu !=0) {
						//Tmp4 = icnt ;
						Tmp4 = in_tutu.size();
						for(k = 1;k <= Tmp4;k++) {
							if (in_tutu [k + -1].building->id() !=i) {
								goto CONSTRUCT_2_CYCLE;
							}
							ag = (rindat [i  - 1][j - 1].start_angle  + rindat [i  - 1][j - 1].end_angle )/2;
							for(l = -2;l <= 2;l += 1) {
								if (in_tutu [k + -1].ewsn  + l == m16(*( &ag )) ) {
									tran = tran * 0.2;
								}
							}
						CONSTRUCT_2_CYCLE:;
						}
					}
					if ((ichk == 1) && (isimyu == 1)) {
						{ cout << " " <<  "24 Scale effect of building"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					if (tikudat [kd + -1].m_dblBuildingAreaGround > tikudat [i + -1].m_dblBuildingAreaGround ) {
						hen = (double )(sqrt(tikudat [i + -1].m_dblBuildingAreaGround ))/(double )(sqrt(tikudat [kd + -1].m_dblBuildingAreaGround ));
					} else {
						hen = 1.0;
					}
					if ((ichk == 1) && (isimyu == 1)) {

						{ cout << " " <<  "25 propagation influence computation"  ;} ;
						{ cout << endl; cout.flush();} ;
					}
					ff = ffpw*tran*hen;
					if (fire [i + -1].tpp <= 0.4) {
						ff = 0;
					}
					fire [kd + -1].rrf  = fire [kd + -1].rrf  + ff;
					rft[kd + -1] = rft[kd + -1] + ff;
					if (rxt[kd + -1]<ff) {
						rxt[kd + -1] = ff;
					}
				LOOP2_CYCLE:;
				}
			LOOP2_EXIT:;
			LOOP1_CYCLE:;
			}
			if ((ichk == 1) && (isimyu == 1)) {

				{ cout << " " <<  "26 Ignition decision to contiguity buildings"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
			Tmp1 = hensu_MP_nkk;
			for(i = 1;i <= Tmp1;i++) {
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "26.1"  ;} ;
					{ cout << " " <<  i  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				if (fire [i + -1].rank_f !=0) {
					goto LOOP3_CYCLE;
				}
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "26.2"  ;} ;
					{ cout << " " <<  i  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				ct1 = fire [i + -1].rrf ;
				ct2 = rft[i + -1] / (para.dt * (1.0 / 60));
				ct3 = rxt[i + -1] / (para.dt * (1.0 / 60));
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "26.3"  ;} ;
					{ cout << " " <<  i  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				ifg = 0;
				cw = 1.0 / para .cww [tikudat [i + -1].k2  + -1];
				cout << "cw " << tikudat[i - 1].id()
				     << " " << cw
				     << " " << tikudat [i + -1].k2 
				     << " " << para .cww [tikudat [i + -1].k2  + -1]
				     << endl;
				ctt1 = para .ctw [1 + -1]*cw;
				ctt2 = para .ctw [2 + -1]*cw;
				ctt3 = para .ctw [3 + -1]*cw;
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "26.4"  ;} ;
					{ cout << " " <<  i  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				cout << "cw1 " << icall << " " << tikudat[i - 1].id() << " " << ct1 << " " << ctt1 << endl;
				cout << "cw2 " << icall << " " << tikudat[i - 1].id() << " " << ct2 << " " << ctt2 << endl;
				cout << "cw3 " << icall << " " << tikudat[i - 1].id() << " " << ct3 << " " << ctt3 << endl;
				if (ct1>=ctt1) {
					ifg = 1;
				}
				if (ct2>=ctt2) {
					ifg = 2;
				}
				if (ct3>=ctt3) {
					ifg = 3;
				}
				if ((ichk == 1) && (isimyu == 1)) {

					{ cout << " " <<  "26.4"  ;} ;
					{ cout << " " <<  i  ;} ;
					{ cout << endl; cout.flush();} ;
				}
				if (ifg!=0) {
					fire [i + -1].rank_f  = 1;
					enslog [i + -1].start  = tnow ;
					switch (ifg) {
					case 1:

						{ cout << " " <<  tikudat [i + -1].id()   ;} ;
						{ cout << " " <<  "It ignited due to 1 product implication spread of fire influence quantity"  ;} ;
						{ cout << " " <<  ct1  ;} ;
						{ cout << " " <<  "  /"  ;} ;
						{ cout << " " <<  para .ctw [1 + -1]  ;} ;
						{ cout << endl; cout.flush();} ;
						break;
					case 2:

						{ cout << " " <<  tikudat [i + -1].id()   ;} ;
						{ cout << " " <<  "It ignited due to 2 unit time spread of fire influence quantity"  ;} ;
						{ cout << " " <<  ct2  ;} ;
						{ cout << " " <<  "  /"  ;} ;
						{ cout << " " <<  para .ctw [2 + -1]  ;} ;
						{ cout << endl; cout.flush();} ;
						break;
					case 3:

						{ cout << " " <<  tikudat [i + -1].id()   ;} ;
						{ cout << " " <<  "It ignited due to 3 unit time maximum spread of fire influence quantity"  ;} ;
						{ cout << " " <<  ct3  ;} ;
						{ cout << " " <<  "  /"  ;} ;
						{ cout << " " <<  para .ctw [3 + -1]  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  "calc_arg_para%at_fprmt_ctw(3)=            "  ;} ;
						{ cout << " " <<  para .ctw [3 + -1]  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  "cw=                     "  ;} ;
						{ cout << " " <<  cw  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  "calc_arg_tikudat(i)%at_bld_dat_buildingCode_k2=          "  ;} ;
						{ cout << " " <<  tikudat [i + -1].k2   ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  "calc_arg_para%at_fprmt_cww(calc_arg_tikudat(i)%at_bld_dat_buildingCode_k2)="  ;} ;
						{ cout << " " <<  para .cww [tikudat [i + -1].k2  + -1]  ;} ;
						{ cout << endl; cout.flush();} ;

						{ cout << " " <<  "rxt(i)=                 "  ;} ;
						{ cout << " " <<  rxt[i + -1]  ;} ;
						{ cout << endl; cout.flush();} ;
					}
				}
			LOOP3_CYCLE:;
			}
			if ((ichk == 1) && (isimyu == 1)) {

				{ cout << " " <<  "27 loop3λ"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
//  			if (isimyu == 1) {
//  				tnow_tmp  = tnow ;
//				icall_tmp  = icall ;
//  			}
		}
		Tmp1 = hensu_MP_nkk;
		for(i = 1;i <= Tmp1;i++) {
			if ((fire [i + -1].rank_f == 1)){  
				{ cout << " " <<  tikudat [i + -1].id()   ;} ;
				{ cout << " " <<  "May burn, is"  ;} ;
				{ cout << endl; cout.flush();} ;
			}
		}
		Tmp1 = hensu_MP_nkk;
		int xx1, xx2, xx3, xx4;
		xx1 = xx2 = xx3 = xx4 = 0;
		for(i = 1;i <= Tmp1;i++) {
			if ((fire [i + -1].rank_f !=0) || (fire [i + -1].tpp !=0.f)) {

				{ cout << " " <<  i  ;} ;
				{ cout << " " <<  tikudat [i + -1].id()   ;} ;
				{ cout << " " <<  fire[i + -1].rank_f   ;} ;
				{ cout << " " <<  fire[i + -1].rank_w   ;} ;
				{ cout << endl; cout.flush();} ;

				{ cout << " " <<  fire[i + -1].tpp   ;} ;
				{ cout << " " <<  fire[i + -1].p   ;} ;
				{ cout << endl; cout.flush();} ;
				switch(fire [i + -1].rank_f ){
				case 1: xx1++; break;
				case 2: xx2++; break;
				case 3: xx3++; break;
				case 4: xx4++; break;
				}
			}
		}
		cout << "burn " << xx1 + xx2 + xx3 + xx4
		     << " " << xx1
		     << " " << xx2
		     << " " << xx3
		     << " " << xx4
		     << endl;
//  		tnow  = tnow_tmp ;
//  		icall  = icall_tmp ;

		{ cout << " " <<  "check write End"  ;} ;
		{ cout << " " <<  icall   ;} ;
		{ cout << " " <<  " Time eye"  ;} ;
		{ cout << endl; cout.flush();} ;

	}

}

