// RescueObjectPool.hxx
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(RESCUEOBJECTPOOL_HXX__INCLUDED) // {
#define RESCUEOBJECTPOOL_HXX__INCLUDED

#include "ObjectPool.hxx"

namespace Rescue
{
	/////////////////////////////////////////////////////////////////////////
	// RescueObjectPool

	class Input;
	class Output;
	/** Gathering of object is managed */
	class RescueObjectPool : public ObjectPool
	{
	private:
		
	protected:
		/** It registers to the mesh FlammableBuilding according to the position */
		int m_meshXBase;	// The coordinate where starting point of the mesh is located
		int m_meshYBase;
		int m_meshXSize;	// Size of 1 mass of mesh
		int m_meshYSize;
		int m_meshXCount;	// The number of masses of mesh
		int m_meshYCount;
		typedef std::vector< std::vector< Objects > > Mesh;
		Mesh m_mesh;
		Objects m_unregisterdBuilding;
		/*
		 * XSize
		 * <--->
		 * +---+---+  <-- YBase
		 * |   |   |
		 * +---+---+
		 * |   |   |
		 * +---+---+
		 *
		 */

	public:
		virtual ~RescueObjectPool();
		RescueObjectPool();

		virtual Object* newObject(TypeId type);

		/**
		* Object is registered.
		* @param	newObject
		* The object which is registered newly. This object delete is done by ObjectPool. Id it is the setting being completed.
		*/
		virtual void add(Object* newObject);

		/** FlammableBuilding which has the possibility of being in circle of specification is enumerated */
		class Enumerator {
			friend RescueObjectPool;
		private:
			RescueObjectPool* m_owner;
			double m_centerX;
			double m_centerY;
			double m_squaredRadius;
			int m_x;
			int m_y;
			int m_xMin;
			int m_xMaz;
			int m_yMin;
			int m_yMaz;
			Objects::const_iterator m_it;
			Enumerator(RescueObjectPool* owner, S32 x, S32 y, S32 r);
			Object* nextCore();
		public:
			Object* next();
		};
		friend Enumerator;
		/** All FlammableBuilding which is in circle of specification are enumerated */
		Enumerator objectsInRound(S32 x, S32 y, S32 r);
		/** The mesh is constructed (re-) */
		void setUpMesh(int meshX, int meshY);
	public:

	private:
		RescueObjectPool(const RescueObjectPool& source);
		RescueObjectPool& operator= (const RescueObjectPool& rhs);
		bool operator== (const RescueObjectPool& rhs) const;
	};
	
	/////////////////////////////////////////////////////////////////////////
} // namespace Rescue

#ifdef NDEBUG
#  include "RescueObjectPool.inl"
#endif

#endif // } !defined(RESCUEOBJECTPOOL_HXX__INCLUDED)

